@@ -1,8 +1,9 @@
 -- Test for v:hlsearch
 
-local helpers = require('test.functional.helpers')(after_each)
-local clear, feed = helpers.clear, helpers.feed
-local feed_command, expect = helpers.feed_command, helpers.expect
+local n = require('test.functional.testnvim')()
+
+local clear, feed = n.clear, n.feed
+local feed_command, expect = n.feed_command, n.expect
 
 describe('v:hlsearch', function()
   setup(clear)
